/*
 * test_io.c
 * Test blocking I/O model which is built on top of PortAudio.
 * Pass input to output.
 *
 * Author: Phil Burk, http://www.softsynth.com
 *
 * This program uses the PortAudio Portable Audio Library.
 * For more information see: http://www.audiomulch.com/portaudio/
 * Copyright (c) 1999-2000 Ross Bencina and Phil Burk
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files
 * (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * Any person wishing to distribute modifications to the Software is
 * requested to send the modifications to the original developer so that
 * they can be incorporated into the canonical version.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR
 * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "portaudio.h"
#include "rwblock.h"
#include <string.h>

/*
** Note that many of the older ISA sound cards on PCs do NOT support
** full duplex audio (simultaneous record and playback).
** And some only support full duplex at lower sample rates.
*/
#define SAMPLE_RATE         (22050)
#define NUM_SECONDS             (5)
#define SAMPLES_PER_FRAME       (2)

/* Select whether we will use floats or shorts. */
#if 1
#define SAMPLE_TYPE  paFloat32
typedef float SAMPLE;
#else
#define SAMPLE_TYPE  paInt16
typedef short SAMPLE;
#endif

/*******************************************************************/
int main(void);
int main(void)
{
	int      i;
	SAMPLE   samples[SAMPLES_PER_FRAME];
	PaError  err;
	RWBL     *rwbl;

	printf("Full duplex sound test using PortAudio and RingBuffers\n");
	fflush(stdout);

/* Open simplified blocking I/O layer on top of PortAudio. */
	err = OpenAudioStream( &rwbl, SAMPLE_RATE, SAMPLE_TYPE,
				     (RWBL_READ_WRITE | RWBL_STEREO) );
	if( err != paNoError ) goto error;

/* Process samples in the foreground. */
	for( i=0; i<(NUM_SECONDS * SAMPLE_RATE); i++ )
	{
	/* Read one frame of data into sample array from audio input. */
		ReadAudioStream( rwbl, samples, 1 );
	/* Write that same frame of data to output. */
	/*	samples[1] = (i&256) * (1.0f/256.0f); /* sawtooth */
		WriteAudioStream( rwbl, samples, 1 );
	}

	CloseAudioStream( rwbl );

	printf("Full duplex sound test complete.\n" );
	fflush(stdout);
	return 0;

error:
	Pa_Terminate();
	fprintf( stderr, "An error occured while using the portaudio stream\n" ); 
	fprintf( stderr, "Error number: %d\n", err );
	fprintf( stderr, "Error message: %s\n", Pa_GetErrorText( err ) );
	return -1;
}
